
import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { DisposableStore, MutableDisposable } from '../../../../base/common/lifecycle.js';
import { observableValue, transaction } from '../../../../base/common/observableInternal/base.js';
import '../../../../base/common/arrays.js';
import '../../../../base/common/event.js';
import { autorun } from '../../../../base/common/observableInternal/autorun.js';
import '../../../../base/common/observableInternal/derived.js';
import '../../../../base/common/cancellation.js';
import { observableFromEvent } from '../../../../base/common/observableInternal/utils.js';
import { OverlayWidgetPositionPreference } from '../../../../editor/browser/editorBrowser.js';
import { MenuWorkbenchToolBar, HiddenItemStrategy } from '../../../../platform/actions/browser/toolbar.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { MenuId } from '../../../../platform/actions/common/actions.js';
import { ActionViewItem } from '../../../../base/browser/ui/actionbar/actionViewItems.js';
import { ACTIVE_GROUP } from '../../../services/editor/common/editorService.js';
import { IEditorService } from '../../../services/editor/common/editorService.service.js';
import { Range } from '../../../../editor/common/core/range.js';
import { append, $, addDisposableGenericMouseMoveListener, reset } from '../../../../base/browser/dom.js';
import { renderIcon } from '../../../../base/browser/ui/iconLabel/iconLabels.js';
import { ThemeIcon } from '../../../../base/common/themables.js';
import { Codicon } from '../../../../base/common/codicons.js';
import { assertType } from '../../../../base/common/types.js';
import { localize } from '../../../../nls.js';
import { navigationBearingFakeActionId, AcceptAction, RejectAction } from './chatEditorActions.js';
import { ChatEditorController } from './chatEditorController.js';
import './media/chatEditorOverlay.css.js';
import { findDiffEditorContainingCodeEditor } from '../../../../editor/browser/widget/diffEditor/commands.js';
import { IChatService } from '../common/chatService.service.js';
import { rcut } from '../../../../base/common/strings.js';
import { IHoverService } from '../../../../platform/hover/browser/hover.service.js';
import { Lazy } from '../../../../base/common/lazy.js';

var ChatEditorOverlayController_1;
let ChatEditorOverlayWidget = class ChatEditorOverlayWidget {
    constructor(_editor, editorService, _hoverService, _chatService, _instaService) {
        this._editor = _editor;
        this._hoverService = _hoverService;
        this._chatService = _chatService;
        this._instaService = _instaService;
        this.allowEditorOverflow = true;
        this._isAdded = false;
        this._showStore = ( new DisposableStore());
        this._entry = observableValue(this, undefined);
        this._navigationBearings = observableValue(this, { changeCount: -1, activeIdx: -1, entriesCount: -1 });
        this._domNode = document.createElement('div');
        this._domNode.classList.add('chat-editor-overlay-widget');
        const progressNode = document.createElement('div');
        progressNode.classList.add('chat-editor-overlay-progress');
        append(progressNode, renderIcon(ThemeIcon.modify(Codicon.loading, 'spin')));
        this._progressNode = append(progressNode, $('SPAN.busy-label'));
        this._domNode.appendChild(progressNode);
        const toolbarNode = document.createElement('div');
        toolbarNode.classList.add('chat-editor-overlay-toolbar');
        this._domNode.appendChild(toolbarNode);
        this._toolbar = _instaService.createInstance(MenuWorkbenchToolBar, toolbarNode, MenuId.ChatEditingEditorContent, {
            telemetrySource: 'chatEditor.overlayToolbar',
            hiddenItemStrategy: HiddenItemStrategy.Ignore,
            toolbarOptions: {
                primaryGroup: () => true,
                useSeparatorsInPrimaryActions: true
            },
            menuOptions: { renderShortTitle: true },
            actionViewItemProvider: (action, options) => {
                const that = this;
                if (action.id === navigationBearingFakeActionId) {
                    return new (class extends ActionViewItem {
                        constructor() {
                            super(undefined, action, { ...options, icon: false, label: true, keybindingNotRenderedWithLabel: true });
                        }
                        render(container) {
                            super.render(container);
                            container.classList.add('label-item');
                            this._store.add(autorun(r => {
                                assertType(this.label);
                                const { changeCount, activeIdx } = that._navigationBearings.read(r);
                                const n = activeIdx === -1 ? '?' : `${activeIdx + 1}`;
                                const m = changeCount === -1 ? '?' : `${changeCount}`;
                                this.label.innerText = ( localize(4525, "{0} of {1}", n, m));
                                this.updateTooltip();
                            }));
                        }
                        getTooltip() {
                            const { changeCount, entriesCount } = that._navigationBearings.get();
                            if (changeCount === -1 || entriesCount === -1) {
                                return undefined;
                            }
                            else if (changeCount === 1 && entriesCount === 1) {
                                return localize(4526, "1 change in 1 file");
                            }
                            else if (changeCount === 1) {
                                return localize(4527, "1 change in {0} files", entriesCount);
                            }
                            else if (entriesCount === 1) {
                                return localize(4528, "{0} changes in 1 file", changeCount);
                            }
                            else {
                                return localize(4529, "{0} changes in {1} files", changeCount, entriesCount);
                            }
                        }
                        onClick(event, preserveFocus) {
                            ChatEditorController.get(that._editor)?.unlockScroll();
                        }
                    });
                }
                if (action.id === AcceptAction.ID || action.id === RejectAction.ID) {
                    return new (class extends ActionViewItem {
                        constructor() {
                            super(undefined, action, { ...options, icon: false, label: true, keybindingNotRenderedWithLabel: true });
                            this._reveal = this._store.add(( new MutableDisposable()));
                        }
                        render(container) {
                            super.render(container);
                            if (action.id === AcceptAction.ID) {
                                const listener = this._store.add(( new MutableDisposable()));
                                this._store.add(autorun(r => {
                                    assertType(this.label);
                                    assertType(this.element);
                                    const ctrl = that._entry.read(r)?.entry.autoAcceptController.read(r);
                                    if (ctrl) {
                                        const r = -100 * (ctrl.remaining / ctrl.total);
                                        this.element.style.setProperty('--vscode-action-item-auto-timeout', `${r}%`);
                                        this.element.classList.toggle('auto', true);
                                        listener.value = addDisposableGenericMouseMoveListener(this.element, () => ctrl.cancel());
                                    }
                                    else {
                                        this.element.classList.toggle('auto', false);
                                        listener.clear();
                                    }
                                }));
                            }
                        }
                        set actionRunner(actionRunner) {
                            super.actionRunner = actionRunner;
                            const store = ( new DisposableStore());
                            store.add(actionRunner.onWillRun(_e => {
                                that._editor.focus();
                            }));
                            store.add(actionRunner.onDidRun(e => {
                                if (e.action !== this.action) {
                                    return;
                                }
                                const d = that._entry.get();
                                if (!d || d.entry === d.next) {
                                    return;
                                }
                                const change = d.next.diffInfo.get().changes.at(0);
                                return editorService.openEditor({
                                    resource: d.next.modifiedURI,
                                    options: {
                                        selection: change && Range.fromPositions({ lineNumber: change.original.startLineNumber, column: 1 }),
                                        revealIfOpened: false,
                                        revealIfVisible: false,
                                    }
                                }, ACTIVE_GROUP);
                            }));
                            this._reveal.value = store;
                        }
                        get actionRunner() {
                            return super.actionRunner;
                        }
                    });
                }
                return undefined;
            }
        });
    }
    dispose() {
        this.hide();
        this._showStore.dispose();
        this._toolbar.dispose();
    }
    getId() {
        return 'chatEditorOverlayWidget';
    }
    getDomNode() {
        return this._domNode;
    }
    getPosition() {
        return { preference: OverlayWidgetPositionPreference.BOTTOM_RIGHT_CORNER };
    }
    showRequest(session) {
        this._showStore.clear();
        const chatModel = this._chatService.getSession(session.chatSessionId);
        const chatRequest = chatModel?.getRequests().at(-1);
        if (!chatRequest || !chatRequest.response) {
            this.hide();
            return;
        }
        this._domNode.classList.toggle('busy', true);
        const message = rcut(chatRequest.message.text, 47);
        reset(this._progressNode, message);
        this._showStore.add(this._hoverService.setupDelayedHover(this._progressNode, {
            content: chatRequest.message.text,
            appearance: { showPointer: true }
        }));
        this._show();
    }
    showEntry(session, activeEntry, next, indicies) {
        this._showStore.clear();
        this._entry.set({ entry: activeEntry, next }, undefined);
        this._showStore.add(autorun(r => {
            const busy = activeEntry.isCurrentlyBeingModified.read(r);
            this._domNode.classList.toggle('busy', busy);
        }));
        this._showStore.add(autorun(r => {
            const value = activeEntry.rewriteRatio.read(r);
            reset(this._progressNode, (value === 0
                ? ( localize(4530, "Generating Edits"))
                : ( localize(4531, "{0}% Applying Edits", Math.round(value * 100)))));
        }));
        this._showStore.add(autorun(r => {
            const entryIndex = indicies.entryIndex.read(r);
            const changeIndex = indicies.changeIndex.read(r);
            const entries = session.entries.read(r);
            let activeIdx = entryIndex !== undefined && changeIndex !== undefined
                ? changeIndex
                : -1;
            let changes = 0;
            for (let i = 0; i < entries.length; i++) {
                const diffInfo = entries[i].diffInfo.read(r);
                changes += diffInfo.changes.length;
                if (entryIndex !== undefined && i < entryIndex) {
                    activeIdx += diffInfo.changes.length;
                }
            }
            this._navigationBearings.set({ changeCount: changes, activeIdx, entriesCount: entries.length }, undefined);
        }));
        this._show();
    }
    _show() {
        const editorWidthObs = observableFromEvent(this._editor.onDidLayoutChange, () => {
            const diffEditor = this._instaService.invokeFunction(findDiffEditorContainingCodeEditor, this._editor);
            return diffEditor
                ? diffEditor.getOriginalEditor().getLayoutInfo().contentWidth + diffEditor.getModifiedEditor().getLayoutInfo().contentWidth
                : this._editor.getLayoutInfo().contentWidth;
        });
        this._showStore.add(autorun(r => {
            const width = editorWidthObs.read(r);
            this._domNode.style.maxWidth = `${width - 20}px`;
        }));
        if (!this._isAdded) {
            this._editor.addOverlayWidget(this);
            this._isAdded = true;
        }
    }
    hide() {
        transaction(tx => {
            this._entry.set(undefined, tx);
            this._navigationBearings.set({ changeCount: -1, activeIdx: -1, entriesCount: -1 }, tx);
        });
        if (this._isAdded) {
            this._editor.removeOverlayWidget(this);
            this._isAdded = false;
            this._showStore.clear();
        }
    }
};
ChatEditorOverlayWidget = ( __decorate([
    ( __param(1, IEditorService)),
    ( __param(2, IHoverService)),
    ( __param(3, IChatService)),
    ( __param(4, IInstantiationService))
], ChatEditorOverlayWidget));
let ChatEditorOverlayController = class ChatEditorOverlayController {
    static { ChatEditorOverlayController_1 = this; }
    static { this.ID = 'editor.contrib.chatEditorOverlayController'; }
    static get(editor) {
        return editor.getContribution(ChatEditorOverlayController_1.ID) ?? undefined;
    }
    constructor(_editor, _instaService) {
        this._editor = _editor;
        this._instaService = _instaService;
        this._overlayWidget = ( new Lazy(
            () => this._instaService.createInstance(ChatEditorOverlayWidget, this._editor)
        ));
    }
    dispose() {
        this.hide();
        this._overlayWidget.rawValue?.dispose();
    }
    showRequest(session) {
        this._overlayWidget.value.showRequest(session);
    }
    showEntry(session, activeEntry, next, indicies) {
        this._overlayWidget.value.showEntry(session, activeEntry, next, indicies);
    }
    hide() {
        this._overlayWidget.rawValue?.hide();
    }
};
ChatEditorOverlayController = ChatEditorOverlayController_1 = ( __decorate([
    ( __param(1, IInstantiationService))
], ChatEditorOverlayController));

export { ChatEditorOverlayController };
